Package: rjags
Version: 4-17
Date: 2025-03-24
Title: Bayesian Graphical Models using MCMC
Authors@R: c(person("Martyn", "Plummer", role=c("aut","cre"),
                    email="martyn.plummer@gmail.com"),
             person("Alexey", "Stukalov", role="ctb"),
             person("Matt", "Denwood", role="ctb"))
Depends: R (>= 2.14.0), coda (>= 0.13)
SystemRequirements: JAGS 4.x.y
URL: https://mcmc-jags.sourceforge.io
Suggests: tcltk
Description: Interface to the JAGS MCMC library.
License: GPL (== 2)
NeedsCompilation: yes
Packaged: 2025-03-24 13:34:40 UTC; martyn
Author: Martyn Plummer [aut, cre],
  Alexey Stukalov [ctb],
  Matt Denwood [ctb]
Maintainer: Martyn Plummer <martyn.plummer@gmail.com>
Repository: CRAN
Date/Publication: 2025-03-24 15:10:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 23:17:14 UTC; unix
Archs: rjags.so.dSYM
